#ifndef PRIORITY_HH
#define PRIORITY_HH

namespace awali{
namespace priority {


/* Concept: priority::value is of any of the type priority::ONE<T>,
 * priority::TWO<T>, priority::THREE<T>, etc.. for some type T (different each
 * time).
 * Since it is more specific, static linkage will preferentially use a function
 * using FIVE<T>, over FOUR<T>, over THREE<T>, etc...
 * Example: assuming existence of functions:
 *  - template<typename P> f(..., ONE<P>) and  
 *  - template<typename P> f(..., TWO<P>)
 * Then a call to f(..., priority::value) will always call the second function.
 * This is useful when multiple algorithms that are doing the same thing are
 * conditional (eg. works only for some weightsets) and we don't wan't to
 * enforce that only one function f(...) exists at the same time.
 * (Cf algos/are-equivalent.hh)
 */
  template<typename T>
  struct  _p {} ;

  template<typename T>
  using ONE = _p<T>;

  template<typename T>
  using TWO = _p<_p<T>>;

  template<typename T>
  using THREE = _p<_p<_p<T>>>;

  template<typename T>
  using FOUR = _p<_p<_p<_p<T>>>>;

  template<typename T>
  using FIVE = _p<_p<_p<_p<_p<T>>>>>;

  FIVE<int> value;

} //end of ns priority
} //end of ns awali


#endif /* PRIORITY_HH */
