#ifndef DYN_MODULES_PROPER_CC
#define DYN_MODULES_PROPER_CC

#include <unordered_map>
#include <awali/common/direction.hh>
#include <awali/dyn/core/automaton.hh>
#include <awali/dyn/core/abstract-ratexp.hh>
#include <awali/dyn/loading/handler.hh>

#include <awali/dyn/modules/proper.hh>

namespace awali { namespace dyn {
  void proper_here(automaton_t aut, param_t dir, bool prune) {
    loading::call3<void, param_t, bool, &proper_here>("proper_here", "proper", aut, dir, prune);
  }

  automaton_t proper(automaton_t aut, param_t dir, bool prune) {
    return loading::call3<automaton_t, param_t, bool, &proper>("proper", "proper", aut, dir, prune);
  }

  bool is_proper(automaton_t aut) {
    return loading::call<bool, &is_proper>("is_proper", "proper", aut);
  }

  bool is_valid(automaton_t aut) {
    return loading::call<bool, &is_valid>("is_valid", "proper", aut);
  }

  automaton_t allow_eps_transition(automaton_t aut) {
    return loading::call<automaton_t, &allow_eps_transition>("allow_eps_transition", "proper", aut);
  }

  automaton_t thompson(ratexp_t exp) {
    std::string stat_ctx = exp->get_context()->sname();
    typedef automaton_t (*bridge_t)(ratexp_t);
    static std::unordered_map<std::string, bridge_t> bridges;
    auto it = bridges.find(stat_ctx);
    if(it == bridges.end()) {
      auto bridge = (bridge_t) loading::get_handler("thompson", "proper", stat_ctx);
      bridges.emplace(stat_ctx, bridge);
      return bridge(exp);
    }
    else
      return it->second(exp);
  }

  automaton_t zpc(ratexp_t exp) {
    std::string stat_ctx = exp->get_context()->sname();
    typedef automaton_t (*bridge_t)(ratexp_t);
    static std::unordered_map<std::string, bridge_t> bridges;
    auto it = bridges.find(stat_ctx);
    if(it == bridges.end()) {
      auto bridge = (bridge_t) loading::get_handler("zpc", "proper", stat_ctx);
      bridges.emplace(stat_ctx, bridge);
      return bridge(exp);
    }
    else
      return it->second(exp);
  }

  automaton_t draw_exp(ratexp_t exp) {
    std::string stat_ctx = exp->get_context()->sname();
    typedef automaton_t (*bridge_t)(ratexp_t);
    static std::unordered_map<std::string, bridge_t> bridges;
    auto it = bridges.find(stat_ctx);
    if(it == bridges.end()) {
      auto bridge = (bridge_t) loading::get_handler("draw_exp", "proper", stat_ctx);
      bridges.emplace(stat_ctx, bridge);
      return bridge(exp);
    }
    else
      return it->second(exp);
  }
}}//end of ns awali::dyn

#endif
