#ifndef AWALI_ALGOS_FACTOR_HH
#define AWALI_ALGOS_FACTOR_HH

#include <awali/sttc/algos/copy.hh>

namespace awali { namespace sttc {


  /*-------.
  | Prefix |
  `-------*/

  template <typename Aut>
  void
  prefix_here(Aut& a)
  {
    for(auto s : a->states())
      a->set_final(s);
  }

  template <typename Aut>
  typename Aut::element_type::automaton_nocv_t
  prefix(const Aut& a)
  {
    auto r = copy(a);
    prefix_here(r);
    return r;
  }

  /*-------.
  | Suffix |
  `-------*/

  template <typename Aut>
  void
  suffix_here(Aut& a)
  {
    for(auto s : a->states())
      a->set_initial(s);
  }

  template <typename Aut>
  typename Aut::element_type::automaton_nocv_t
  suffix(const Aut& a)
  {
    auto r = copy(a);
    suffix_here(r);
    return r;
  }

  /*-------.
  | Factor |
  `-------*/

  template <typename Aut>
  void
  factor_here(Aut& a)
  {
    prefix_here(a);
    suffix_here(a);
  }

  template <typename Aut>
  typename Aut::element_type::automaton_nocv_t
  factor(const Aut& a)
  {
    auto r = copy(a);
    factor_here(r);
    return r;
  }

}}//end of ns awali::stc

#endif // !AWALI_ALGOS_FACTOR_HH
