#ifndef AWALI_ALGOS_SUB_AUTOMATON_HH
# define AWALI_ALGOS_SUB_AUTOMATON_HH

# include <set>

namespace awali { namespace sttc {


  /*-----------------.
  | sub-automaton).  |
  `-----------------*/

  template <typename Aut, typename Pred>
  inline
  void
  sub_automaton(Aut& aut, Pred keep_state)
  {
    std::vector<state_t> to_erase;
    for(auto s : aut->states())
      if(!keep_state(s))
	to_erase.emplace_back(s);
    for(auto s : to_erase)
      aut->del_state(s);
  }

  template <typename Aut>
  inline
  void
  sub_automaton(Aut& aut, const std::set<state_t>& sts)
  {
    std::vector<state_t> to_erase;
    auto it=sts.begin();
    for(auto s : aut->all_states())
      if(it == sts.end() || *it != s) {
	if(s!= aut->pre() && s!=aut->post())
	  to_erase.emplace_back(s);
      }
      else
	++it;
    for(auto s : to_erase)
      aut->del_state(s);
  }

}}//end of ns awali::stc

#endif // !AWALI_ALGOS_SUB_AUTOMATON_HH
