#ifndef AWALI_ALGOS_UNION_HH
# define AWALI_ALGOS_UNION_HH

# include <unordered_map>

#include <awali/sttc/algos/copy.hh>
#include <awali/sttc/algos/product.hh> // join_automata
#include <awali/sttc/core/mutable-automaton.hh>

namespace awali { namespace sttc {

  /*----------.
  | union_a.  |
  `----------*/

  /// Merge transitions of \a b into those of \a res.
  ///
  /// \pre AutIn <: AutOut.
  template <typename A, typename B>
  inline
  A&
  union_here(A& res, const B& b)
  {
    sttc::copy_into(b, res);
    return res;
  }

  /// Union of two automata.
  template <typename A, typename B>
  inline
  auto
  union_a(const A& lhs, const B& rhs)
    -> decltype(join_automata(lhs, rhs))
  {
    auto res = join_automata(lhs, rhs);
    union_here(res, lhs);
    union_here(res, rhs);
    return res;
  }

}}//end of ns awali::stc

#endif // !AWALI_ALGOS_UNION_HH
